/*
  Implementation of associative arrays (indexed by string) that store
  integer values.

  Diego Zamboni, Mar 7, 1997.

  */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "assoc_array_int.h"

char * last_hash_string = NULL;
unsigned last_hash_value;

/* Taken from K&R, 6.6 */
unsigned hash(char *s) {
    unsigned hashval;

    for(hashval=0; *s!='\0'; s++)
	hashval=*s+31*hashval;
    return hashval % HASHSIZE;
}


/* Taken from K&R, 6.6 */
assoc_array_int_node assoc_array_int_lookup(assoc_array_int as, char *ind) {
    assoc_array_int_node np;

    /* I tried caching last index value.  Instead of speeding things up for
       ifile database reading/writing, it actually slows things down */

    for(np=as[hash(ind)]; np!=NULL; np=np->next) {
	if(strcmp(ind, np->index)==0)
	    return np;   /* Found */
    }
    return NULL;     /* Not found */
}


void assoc_array_int_init(assoc_array_int as) {
    int i;
    for(i=0; i<HASHSIZE; i++)
	as[i]=NULL;
}


/* NOTE: index string, 'ind', given to function is duplicated, so the string
   which is passed may later be freed without need for concern */
/* Taken largely from K&R, 6.6 */
void assoc_array_int_set(assoc_array_int as, char *ind, int val) {
    assoc_array_int_node np;
    unsigned hashval;

    if((np=assoc_array_int_lookup(as, ind))==NULL) { /* Not found */
	np=(assoc_array_int_node)malloc(sizeof(*np));
	hashval=hash(ind);
	/* Set index */
	np->index=strdup(ind);
	/* Insert at the beginning of the list */
	np->next=as[hashval];
	as[hashval]=np;
    }
    /* Set new value */
    np->value=val;
}


int assoc_array_int_get(assoc_array_int as, char *ind) {
    assoc_array_int_node np;
    if((np=assoc_array_int_lookup(as, ind))!=NULL) {
	return np->value;
    }
    else {
	return 0;
    }
}


void assoc_array_int_add(assoc_array_int as, char *ind, int val) {
    assoc_array_int_set(as,ind,val);
}
